@@ -24,27 +24,50 @@ import Exceptions
 import ID
 import Enum
 
+#
+# XXX FIXME:
+#     We should fetch the properies from Node and Entry classes. Only
+#     those are the allowed symbols inside a filter (a subset of them
+#     all: children and parent should be hidden ...)
+#
+def help() :
+    return "depth, id"
+
 class Node(object) :
     def __init__(self) :
         self.__children  = []
 #        self.__iterator  = 0
         self.__parent    = None
-        self.__allflags  = Enum.Enum('visible',
-                                     'collapsed')
+        self.__allflags  = Enum.Enum('visible', 'collapsed')
         self.__flags     = []
+        self.__depth     = 0
 
         debug("Node `" + str(self) + "' created successfully")
 
+    def _update_depth(self, value) :
+        assert(value >= 0)
+        self.__depth = value
+        for i in self.__children :
+            i._update_depth(value + 1)
+
     def parent_get(self) :
         return self.__parent
     def parent_set(self, node) :
         self.__parent = node
+        self._update_depth(node.depth + 1)
 
-    parent = property(parent_get, parent_set)
+    parent = property(parent_get, parent_set, None, None)
 
-    def children(self) :
+    def depth_get(self) :
+        return self.__depth
+
+    depth = property(depth_get, None, None, None)
+
+    def children_get(self) :
         return self.__children
 
+    children = property(children_get, None, None, None)
+
 #    # Add/Remove a node based on id
 #    def child(self, i = 0, node = None) :
 #       assert(i >= 0)
@@ -69,31 +92,33 @@ class Node(object) :
     # Add a child node based on object
     def add(self, node) :
         assert(node != None)
-        i     = 0
+
         found = False
         for j in self.__children :
             if (j == node) :
                 found = True
                 break
-            i = i + 1
         assert(found == False)
+
         self.__children.append(node)
         node.__parent = self
+        node._update_depth(self.__depth + 1)
         debug("Node `" + str(node) + "' added to node `" + str(self) + "'")
 
     # Remove a child node based on object
     def remove(self, node) :
         assert(node != None)
-        i     = 0
+
         found = False
         for j in self.__children :
             if (j == node) :
                 found = True
                 break
-            i = i + 1
         assert(found == True)
+
         self.__children.remove(node)
         node.__parent = None
+        node._update_depth(0)
         debug("Node `" + str(node) + "' removed to node `" + str(self) + "'")
 
     # Iterator related methods
@@ -133,13 +158,13 @@ class Node(object) :
         debug("Node id is `" + str(i) + "'")
         return i
 
-    id = property(id_get, None)
+    id = property(id_get, None, None, None)
 
     def flags_get(self) :
         return self.__flags
 
     def flags_set(self, flags) :
-        assert(type(flags) == list)
+        assert(isinstance(flags, list))
 
         for f in flags :
             try:
@@ -148,7 +173,7 @@ class Node(object) :
                 raise Exceptions.UnknownEnum(str(f))
         self.__flags = flags
 
-    flags = property(flags_get, flags_set)
+    flags = property(flags_get, flags_set, None, None)
 
 # Test
 if (__name__ == '__main__') :
@@ -171,32 +196,51 @@ if (__name__ == '__main__') :
 
     print(str(root.id))
     assert(root.id == ID.ID("0"))
+    assert(root.depth == 0)
 
     print(str(e1.id))
     assert(e1.id   == ID.ID("0.1"))
+    assert(e1.depth == 1)
 
     print(str(e11.id))
     assert(e11.id  == ID.ID("0.1.1"))
+    assert(e11.depth == 2)
 
     print(str(e12.id))
     assert(e12.id  == ID.ID("0.1.2"))
+    assert(e12.depth == 2)
 
     print(str(e121.id))
     assert(e121.id == ID.ID("0.1.2.1"))
+    assert(e121.depth == 3)
 
     print(str(e13.id))
     assert(e13.id  == ID.ID("0.1.3"))
+    assert(e13.depth == 2)
 
     print(str(e2.id))
     assert(e2.id   == ID.ID("0.2"))
+    assert(e2.depth == 1)
 
     print(str(e21.id))
     assert(e21.id  == ID.ID("0.2.1"))
+    assert(e21.depth == 2)
 
     assert(e21.id  == ID.ID("0.2.1"))
     assert(e2.id   == ID.ID("0.2"))
     assert(e13.id  == ID.ID("0.1.3"))
     assert(e121.id == ID.ID("0.1.2.1"))
+
+    e12.remove(e121)
+    assert(e121.depth == 0)
+    e12.add(e121)
+    assert(e121.depth == 3)
+
+    e12.remove(e121)
+    assert(e121.depth == 0)
+    e121.parent = e12
+    assert(e121.depth == 3)
+
     assert(e12.id  == ID.ID("0.1.2"))
     assert(e11.id  == ID.ID("0.1.1"))
     assert(e1.id   == ID.ID("0.1"))
